<?php

/*
 * Picscript
 * Written by Mark Kamichoff <prox@prolixium.com>
 */

$version = "1.9";

// Initialize some global-ish vars
$descriptions=array();
$title="(error opening var/descriptions.txt)";
$default_descriptions=1;

$numfiles=-1;
$files=array();

function ifmis($val) {
	if(isset($val)) { return $val; }
	else { return "[Missing]"; }
}

/* This calculates which page an image is on */
function whichpage($id) {
	global $numfiles, $npp;

	$numpages = ceil($numfiles / $npp);
	for($i=1;$i<$numpages;$i++) {
		if(($id >= (($i-1) * $npp)) && ($id <= ($i * $npp))) {
			return $i;
		}
	}

	/* If we didn't get anywhere, must be last page! */
	return $numpages;
}

function xmlheading($title) {
	print <<<EOT
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html 
     PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">

EOT;
	print "<head>\n";
	print "<link rel=\"alternate stylesheet\" type=\"text/css\" href=\"lib/dark.css\" title=\"Dark\" />\n";
	print "<link rel=\"stylesheet\" type=\"text/css\" href=\"lib/bright.css\" title=\"Bright\" />\n";
	print "<title>{$title}</title>\n";
	print "</head>\n";

	print "<body>\n";
}
	

function read_the_dir() {
	global $files, $numfiles;
	$tempfiles = array();

	// Read all jpg and png files in, now
	if($dh = opendir(".")) {
	
		while(false !== ($file = readdir($dh))) {
			if(preg_match("/.+\.jpg/", $file) ||
					preg_match("/.+\.JPG/", $file) ||
					preg_match("/.+\.jpeg/", $file) ||
					preg_match("/.+\.JPEG/", $file) ||
					preg_match("/.+\.png/", $file) ||
					preg_match("/.+\.PNG/", $file)) {

				$tempstat = stat($file);
				
				/* sort by last inode modification time */
				$tempfiles[$file] = $tempstat['mtime'];
			}
		}
	
		/* do the sort */
		asort($tempfiles);
		$files = array_keys($tempfiles);
		$numfiles = count($files);
		closedir($dh);

	} else {
		print "Error opening directory.";
		die;
	}
}

function do_cookies() {
	// Mess with cookies, first
	if($_REQUEST['npp'] > 0) {	// If we specified
		// Save the cookie
		setcookie("ProlixiumPicscriptNPP", $_REQUEST['npp'],
				time()+86400);	// Expire in 24 hours
		// Set the var
		$npp = $_REQUEST['npp'];
	} else if($_COOKIE['ProlixiumPicscriptNPP']) {	// We got a cookie
		// Set the var
		$npp = $_COOKIE['ProlixiumPicscriptNPP'];
	} else {
		// Fall back to default
		$npp = 15;
	}

	return $npp;
}

function end_page($npp, $numfiles, $version) {
	
	print "<form method=\"post\" action=\"{$_SERVER['PHP_SELF']}\">\n";
	print "<table style=\"width: 100%\">";
	print "\n<tr><td class=\"red\" style=\"vertical-align: middle; width: 35%\">";
	
	print "<a href=\"";
	if($_REQUEST['image']) {
		$retpage = whichpage($_REQUEST['image']);
		print "?page={$retpage}\">Thumbnail view, page $retpage</a>\n";
	} else {
		print "../";
		print "\">Up one directory</a>";
	}
	print " ($numfiles total images)</td>\n";
	
	print "<td class=\"grey\" style=\"text-align: center; vertical-align: middle; width: 30%\">";
	
	// Display small form, to set $npp
	print "Display <input style=\"width: 4em;\" type=\"text\" name=\"npp\" value=\"{$npp}\" /> pictures per page.\n";
	
print <<<HERE
		</td>
		<td class="red" style="vertical-align: middle; text-align: right; width: 35%"><a href="http://validator.w3.org/check?uri=referer"><img src="lib/xhtml.png" alt="Valid XHTML 1.0 Strict" style="border: 0px" /></a><a href="http://jigsaw.w3.org/css-validator/"><img src="lib/css.png" alt="Valid CSS!" style="border: 0px" /></a></td>
	</tr>
</table>
</form>
	
</body>
</html>

HERE;

}

function read_descriptions() {
	// Open the file, optionally
	// File has format like this:
	// -----------------------------------------------------
	// Title of photo album
	// (default_descriptions|no_default_descriptions)
	// photo.jpg,description of photo.jpg
	// photo2.jpg,description of photo2.jpg
	// [etc...]
	// -----------------------------------------------------
	// Reads it into an array, and stuff

	global $descriptions, $title, $default_descriptions;

	if($fd = fopen("var/descriptions.txt", 'r')) {
	
		// Title
		$title = rtrim(fgets($fd));
		if(rtrim(fgets($fd)) == "default_descriptions") {
			$default_descriptions=1;
		} else {
			$default_descriptions=0;
		}
		
		while($line = fgets($fd)) {
			if(preg_match("/([^,]+),(.+)/", $line, $matches)) {
				$descriptions[$matches[1]] = $matches[2];
			}
		}
	
		fclose($fd);
	}
}

// Read the dir, get $numfiles and $files
read_the_dir();

// Do cookies!
$npp = do_cookies();

// Check if we're supposed to display an individual image
if($_REQUEST['image']) {

	/* Sanity check */
	if($_REQUEST['image'] < 1) {
		header("content-type: text/plain");
		print "Error: Invalid index.\n";
		exit;
	}

	read_descriptions();

	/* Set a var */
	$file_name = $files[$_REQUEST['image']-1];
	$file_name_p = $files[$_REQUEST['image']-2];
	$file_name_n = $files[$_REQUEST['image']];

	if($descriptions[$file_name]) {
		xmlheading("$title - {$descriptions[$file_name]}");
	} else {
		xmlheading("$title - $file_name");
	}

	/* Top heading table, w/next/prev buttons */
	print "<table style=\"width: 100%; margin-left: auto; margin-right: auto\">\n";
	print "\t<tr>\n";

	/* Prev */
	print "\t\t<td class=\"nextprev\">";
	if($_REQUEST['image'] > 1) {
		print "<a href=\"?image=".($_REQUEST['image']-1)."\"><img class=\"arrows\" src=\"lib/prev.png\" title=\"Previous Image\" alt=\"previous\" /></a>";
	} else {
		print "&nbsp;";
	}
	print "</td>\n";

	/* Title and stuff */
	print "\t\t<td rowspan=\"2\" style=\"vertical-align: middle; text-align: center; width: 60%\">\n";
	print "\t\t\t<div style=\"font-size: 24px\">";
	if($descriptions[$file_name]) {
		print $descriptions[$file_name];
	} else {
		print $file_name;
	}
	print "</div>\n";
	print "\t\t\t<div>{$title} ";
	print " ({$_REQUEST['image']}/{$numfiles})";
	print "</div>\n";
	print "\t\t\t</td>\n";
	
	/* Next */
	print "\t\t<td class=\"nextprev\">";
	if($_REQUEST['image'] == count($files)) {
		print "&nbsp;";
	} else {
		print "<a href=\"?image=".($_REQUEST['image']+1)."\"><img class=\"arrows\" src=\"lib/next.png\" title=\"Next Image\" alt=\"next\" /></a>";
	}

	print "</td>\n";

	print "</tr>\n";
	print "<tr>\n";
	
	/* Prev caption, if we have one */
	print "\t\t<td class=\"nextprev\">";
	if($_REQUEST['image'] > 1) {
		if($descriptions[$file_name_p]) {
			print $descriptions[$file_name_p];
		} else {
			print $file_name_p;
		}
	} else {
		print "&nbsp;";
	}
	print "</td>\n";

	/* Next caption, if we have one */
	print "\t\t<td class=\"nextprev\">";
	if($_REQUEST['image'] >= 1) {
		if($descriptions[$file_name_n]) {
			print $descriptions[$file_name_n];
		} else {
			print $file_name_n;
		}
	} else {
		print "&nbsp;";
	}
	print "</td>\n";

	print "</tr>\n";
	print "</table>\n";

	print "<table style=\"margin-left: 0%; margin-right: 0%; width: 100%;\">\n";
	print "\t<tr>\n";

	print "\t\t<td style=\"width: 25%; vertical-align: top; padding-top: 32px;\">\n";
	print "\t\t\t<div style=\"text-align: left; font-weight: bold;\">Image Details</div>\n";

	print "\t\t\t<ul style=\"font-size: 12px;\">\n\n";
	/* Normal data */
	print "\t\t\t\t<li>File name: $file_name</li>\n";
	print "\t\t\t\t<li>Description: ".ifmis($descriptions[$file_name])."</li>\n";
	print "\t\t\t\t<li>file(1): ".htmlspecialchars(`file -b $file_name`)."</li>\n";
	print "\t\t\t</ul>\n\n";

	/* EXIF ul */
	print "\t\t\t<div style=\"text-align: left; font-weight: bold;\">EXIF Data</div>\n";
	print "\t\t\t<ul style=\"font-size: 12px;\">\n\n";

	/* EXIF data */
	/* Read it */
	if(preg_match("/.+\.(jpg|jpeg)$/i", $file_name)) {
		if($exif = exif_read_data($file_name, 0, true)) {
			print "\t\t\t\t<li>Timestamp: ".ifmis($exif['IFD0']['DateTime'])."</li>\n";
			print "\t\t\t\t<li>Dimensions: {$exif['COMPUTED']['Width']} x {$exif['COMPUTED']['Height']}</li>\n";
			print "\t\t\t\t<li>Camera make: ".ifmis($exif['IFD0']['Make'])."</li>\n";
			print "\t\t\t\t<li>Camera model: ".ifmis($exif['IFD0']['Model'])."</li>\n";
			print "\t\t\t\t<li>Exposure time: ".ifmis($exif['EXIF']['ExposureTime'])." sec</li>\n";
			print "\t\t\t\t<li>Exposure mode: ";
			if(isset($exif['EXIF']['ExposureMode'])) {
				switch($exif['EXIF']['ExposureMode']) {
					case 0:
						print "Auto";
						break;
					case 1:
						print "Manual";
						break;
					case 2:
						print "Auto bracket";
						break;
					default:
						print "(unknown)";
						break;
					
				}
			} else { print "[Missing]"; }
			print "</li>\n";
			print "\t\t\t\t<li>Shutter speed: ".ifmis($exif['EXIF']['ShutterSpeedValue'])."</li>\n";
			print "\t\t\t\t<li>White balance: ";
			if(isset($exif['EXIF']['WhiteBalance'])) {
				if($exif['EXIF']['WhiteBalance']) {
					print "Manual";
				} else { print "Auto"; }
			} else {
				print "[Missing]";
			}
			print "</li>\n";
			print "\t\t\t\t<li>Flash: ";
			if(!empty($exif['EXIF']['Flash'])) {
				switch ($exif['EXIF']['Flash']) {
					case hexdec("0x0001"):
						print "Fired";
						break;
					case hexdec("0x0005"):
						print "Strobe return light not detected";
						break;
					case hexdec("0x0007"):
						print "Strobe return light detected";
						break;
					case hexdec("0x0009"):
						print "Fired";
						break;
					case hexdec("0x000D"):
						print "Fired, compulsory, no return light";
						break;
					case hexdec("0x000F"):
						print "Fired, compulsory, return light";
						break;
					case hexdec("0x0010"):
						print "None, compulsory";
						break;
					case hexdec("0x0018"):
						print "None, auto";
						break;
					case hexdec("0x0019"):
						print "Fired, auto";
						break;
					case hexdec("0x001D"):
						print "Fired, auto, no return light";
						break;
					case hexdec("0x001F"):
						print "Fired, auto, return light";
						break;
					case hexdec("0x0020"):
						print "No function";
						break;
					case hexdec("0x0041"):
						print "Fired, red-eye";
						break;
					case hexdec("0x0045"):
						print "Fired, red-eye, no return light";
						break;
					case hexdec("0x0047"):
						print "Fired, red-eye, return light";
						break;
					case hexdec("0x0049"):
						print "Fired, compulsory, red-eye";
						break;
					case hexdec("0x004D"):
						print "Fired, compulsory, red-eye, no return light";
						break;
					case hexdec("0x004F"):
						print "Fired, compulsory, red-eye, return light";
						break;
					case hexdec("0x0059"):
						print "Fired, auto, red-eye";
						break;
					case hexdec("0x005D"):
						print "Fired, auto, no return light, red-eye";
						break;
					case hexdec("0x005F"):
						print "Fired, auto, return light, red-eye";
						break;
					default:
						print "(unknown)";
						break;
				}
			} else {
				print "None";
			}
			print "</li>\n";
		} else {
			print "\t\t\t\t<li>Warning: No EXIF data for {$file_name}.</li>\n";
		}
	} else {
		print "\t\t\t\t<li>Notice: Unsupported in non-JPEG images.</li>\n";
	}

	print "\t\t\t</ul>\n";
	print "\t\t</td>\n";

	/* The image */
	print "\t\t<td style=\"width: 25%; text-align: center; vertical-align: middle;\">";
	print "<a href=\"{$file_name}\"><img src=\"previews/{$file_name}\" title=\"Click for full-size {$file_name}\" alt=\"{$file_name}\" /></a>\n";
	print "</td>\n";

	print "\t</tr>\n";

	print "</table>\n";

	// All done
	end_page($npp,$numfiles,$version);
} else { 
	
	read_descriptions();

	// Page number, optional
	$page = 0;
	if($_REQUEST['page']) {
		if($_REQUEST['page']-1 > 0) {
			// This used to be <=
			if(($_REQUEST['page']-1)*$npp < $numfiles) {
				$page = $_REQUEST['page']-1;
			}
		}
	}
	
	// Do heading, now
	$maxpage=ceil($numfiles/$npp);
	xmlheading($title, $page+1, $maxpage);
	
	print "\n<table style=\"width: 100%; margin-left: auto; margin-right: auto\">\n";
	
	print "\t<tr>\n";
	print "\t\t<td class=\"nextprev\">";
	
	if($page != 0) {
		print "<a href=\"?page=".($page)."\"><img class=\"arrows\" src=\"lib/prev.png\" title=\"Previous Page\" alt=\"previous\" /></a>";
	} else {
		print "&nbsp;";
	}
	
	print "</td>\n";
	
	print "\t\t<td style=\"vertical-align: middle; text-align: center; width: 60%\">";
	print "<div style=\"font-size: 24px\">$title (Page ".($page+1)."/$maxpage)</div>";
	print "</td>\n";
	
	print "\t\t<td class=\"nextprev\">";
	
	if(($npp*($page+1) <= $numfiles) && ceil($numfiles/$npp) > $page+1) {
		print "<a href=\"?page=".($page+2)."\"><img class=\"arrows\" src=\"lib/next.png\" title=\"Next Page\" alt=\"next\" /></a>";
	} else {
		print "&nbsp;";
	}
	
	print "</td>\n";
	print "\t</tr>\n";
	
	print "</table>\n\n";
	
	// Some conditions
	if($npp*($page+1) > $numfiles) {
		$max = $numfiles;
	} else {
		$max = ($npp)*($page+1);
	}
	
	// Display table, will do CSS and stuff later
	print "<table style=\"width: 100%\">\n";
	print "\t<tr>\n";
	
	for($numb = $page*$npp; $numb < $max; $numb++) {
		if(!(($numb-($page*$npp))%5) && $numb!=$page*$npp) {
			print "\t</tr>\n";
			print "\t<tr>\n";
		}
	
		print "\t\t<td class=\"thumb\" style=\"text-align: center; vertical-align: top; width: 20%\">\n";
		print "\t\t\t<div style=\"text-align: center;\">\n";
		print "\t\t\t\t<a href=\"";
		
		// This will allow us to do next/prev on the individual
		print "?image=".($numb+1)."\"><img src=\"thumbs/{$files[$numb]}\" title=\"Click to enlarge {$files[$numb]}\" alt=\"{$files[$numb]}\" /></a>\n";
		print "\t\t\t</div>\n";
		print "\t\t\t<div class=\"caption\">";
	
		// Description
		if($descriptions[$files[$numb]]) {
			print $descriptions[$files[$numb]];
		} else {
			if($default_descriptions) {
				print $files[$numb];
			} else {
				print "No label";
			}
		}
	
		print "</div>\n";
		print "\t\t</td>\n\n";
	}
	
	print "\t</tr>\n";
	
	print "</table>\n";

	end_page($npp,$numfiles,$version);
}

?>
