#!/bin/sh
# Picscript
# Mark Kamichoff <prox@prolixium.com>



VERSION=1.9
CONVERT="$(which convert)"

# Check if we have ImageMagick
if ! test -x "$CONVERT"; then
	echo "Looks like you don't have ImageMagick installed, aborting."
	exit 1
fi

# Ripped from http://www.die.net/doc/linux/abs-guide/contributed-scripts.html
FOUND=0
for filename in *	# Traverse all files in directory.
do
	echo "$filename" | grep -q " "         # Check whether filename contains spaces
	if [ $? -eq $FOUND ]; then
		fname=$filename                      # Strip off path.
		n=`echo $fname | sed -e "s/ /_/g"`   # Substitute underscore for blank.
		n=`echo $n | sed -e "s/&/and/g"`   # Substitute & for "and"
		mv "$fname" "$n"	             # Do the actual renaming.
	fi
done 

# tolower()
for x in `ls -b1`
	do
	if [ ! -f $x ]; then
		continue
	fi
	if test '$x = Makefile'; then
		continue
	fi
	convert=`echo $x | tr '[A-Z]' '[a-z]'`
	if [ $convert != $x ]; then
		mv -i $x $convert
	fi
done

# Make dirs
if [ ! -d previews ]; then
	echo -n "Creating previews dir: ";
	mkdir previews
	echo "done."
fi
if [ ! -d thumbs ]; then
	echo -n "Creating thumbs dir: ";
	mkdir thumbs
	echo "done."
fi
if [ ! -d var ]; then
	echo -n "Creating var dir: ";
	mkdir var
	echo "done."
fi

# Convert all image files
for TYPE in "JPG" "jpg" "JPEG" "jpeg" "PNG" "png"
do
	echo -n "Processing "$TYPE" files: ";

	THM=0
	PVW=0
	for x in `ls -b *.$TYPE 2> /dev/null`
		do
		if [ ! -f thumbs/$x ]; then
			# Make thumb
			convert -resize 160 $x thumbs/$x
			echo -n .
	
			THM=`expr $THM + 1`
		fi
	
		if [ ! -f previews/$x ]; then
	
			# Make preview
			convert -resize 800 $x previews/$x
			echo -n .
	
			PVW=`expr $PVW + 1`
		fi
	done

	if [ "$(expr $THM + $PVW)" != "0" ]; then
#		echo " ($THM thumbs, $PVW previews)"
		echo " done."
	else
		echo "done."
	fi

#	echo " * Created $THM $TYPE thumbnails.";
#	echo " * Created $PVW $TYPE previews.";
done

# Make blank descriptions, if it doesn't exist already
if [ ! -f var/descriptions.txt ]; then
	echo "Images in \"$(basename $PWD)\"" > var/descriptions.txt
	echo "default_descriptions" >> var/descriptions.txt
fi
